/*

    ivscale -- a C/Python module providing a gtk vscale with the 
               maximum at the top

    Copyright (C) 1999  Eric S. Tiedemann

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Contact: Eric S. Tiedemann, est@hyperreal.org

*/

/*
  
  prettify names (gtk_ivscale_new(), etc)
  handle set_draw_value() and set_value_pos
  perhaps inherit from gtk_vscale instead?
  signal error if null adjustment given
  make correct!

*/

#include <gtk/gtk.h>
#include <stdio.h>

#include "ivscale.h"

static void
ivscale_class_init (IvscaleClass *class)
{
  GtkObjectClass *object_class;

  object_class = (GtkObjectClass*) class;
}

static void
ivscale_init (Ivscale *ivs)
{
}

void
cb(GtkAdjustment *a, struct _Ivscale *me)
{
  if (me->l)
    {
      char buf[100];
      snprintf(buf, sizeof(buf), "%.2f", me->adj->upper - a->value);
      gtk_label_set_text(GTK_LABEL(me->l), buf);
    }

  gtk_adjustment_set_value(me->adj, me->adj->upper - a->value);
}

/* set adj's value to v without triggering the signal handler
   registered with id sigid */
static void
diddle_adjustment(GtkObject *adj, int sigid, double v)
{
  gtk_signal_handler_block(adj, sigid);
  gtk_adjustment_set_value(GTK_ADJUSTMENT(adj), v);
  gtk_signal_handler_unblock(adj, sigid);
  gtk_signal_emit_by_name(adj, "changed");
}

void
cb1(GtkAdjustment *a, struct _Ivscale *me)
{
  if (me->l)
    {
      char buf[100];
      snprintf(buf, sizeof(buf), "%.2f", a->value);
      gtk_label_set_text(GTK_LABEL(me->l), buf);
    }

  diddle_adjustment(GTK_OBJECT(me->adj1), 
		    me->sigid, 
		    a->upper - a->value);
}

GtkWidget*
ivscale_new (GtkAdjustment *a)
{
  struct _Ivscale *me;
  GtkWidget *s;
  char buf[100];

  GTK_ADJUSTMENT(a);
  me = gtk_type_new (ivscale_get_type ());
  sprintf(me->fmt, "%%.1f");
  snprintf(buf, sizeof(buf), me->fmt, a->value);

  me->l = gtk_label_new(buf);
  gtk_widget_show(me->l);
  gtk_box_pack_start(GTK_BOX(me), me->l, 0, 0, 0);

  me->adj = a;
  me->adj1 = GTK_ADJUSTMENT(gtk_adjustment_new(a->upper - a->value,
					       a->lower, a->upper, 
					       a->step_increment,
					       a->page_increment, 
					       a->page_size));

  me->sigid = 
    gtk_signal_connect(GTK_OBJECT(me->adj1), "value_changed", cb, me);
  gtk_signal_connect(GTK_OBJECT(me->adj), "value_changed", cb1, me);

  s = gtk_vscale_new(me->adj1);
  gtk_scale_set_draw_value(GTK_SCALE(s), FALSE);
  gtk_widget_show(s);

  gtk_box_pack_start(GTK_BOX(me), s, 1, 1, 0);

  return GTK_WIDGET (me);
}

void
ivscale_set_digits(Ivscale *i, gint d)
{
  snprintf(i->fmt, sizeof(i->fmt), "%%.%df", d);
  if (i->l)
    {
      char buf[100];
      snprintf(buf, sizeof(buf), "%.2f", i->adj->upper - i->adj1->value);
      gtk_label_set_text(GTK_LABEL(i->l), buf);
    }
}

guint
ivscale_get_type ()
{
  static guint ivs_type = 0;

  if (!ivs_type)
    {
      GtkTypeInfo ivs_info =
      {
        "Ivscale",
        sizeof (Ivscale),
        sizeof (IvscaleClass),
        (GtkClassInitFunc) ivscale_class_init,
        (GtkObjectInitFunc) ivscale_init,
        (GtkArgSetFunc) NULL,
        (GtkArgGetFunc) NULL
      };

      ivs_type = gtk_type_unique (gtk_vbox_get_type (), &ivs_info);
    }

  return ivs_type;
}

//#define TEST
#ifdef TEST

void
bar(GtkWidget *w, void *v)
{
  fprintf(stderr, "%f\n", GTK_ADJUSTMENT(w)->value);
}

int
main(int argc, char **argv)
{
  GtkWidget *w;
  GtkWidget *i;
  GtkWidget *l;
  GtkObject *a;

  gtk_init(&argc, &argv);

  w  = gtk_window_new(GTK_WINDOW_TOPLEVEL);
  gtk_widget_show(w);
  gtk_signal_connect(GTK_OBJECT(w), "delete_event", gtk_main_quit, 0);

  a = gtk_adjustment_new(1, 0, 1, .01, .1, 0);
  gtk_signal_connect(a, "value_changed", bar, 0);
  i = ivscale_new(GTK_ADJUSTMENT(a));
  ivscale_set_digits(IVSCALE(i), 2);
  gtk_widget_show(i);
  gtk_container_add(GTK_CONTAINER(w), i);

  gtk_main();
}
#endif
